/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import javax.annotation.Nullable;
import net.java.truelicense.core.codec.Codec;

/**
 * Defines authentication services.
 *
 * @author Christian Schlichtherle
 */
public interface Authentication extends AuthenticationParametersProvider {

    /**
     * Encodes and signs the given {@code artifact} in the provided
     * repositoryProvider and returns an artifactory for it.
     * After calling this method, the repositoryProvider typically gets encoded for
     * future decoding and verification.
     * <p>
     * It is an implementation detail whether or not signing {@code null} is
     * supported or not.
     *
     * @param  codec the codec for encoding the artifact.
     * @param  repository the repository for encoding the artifact to.
     * @param  artifact the artifact to sign.
     */
    Artifactory sign(Codec codec, Repository repository, @Nullable Object artifact)
    throws Exception;

    /**
     * Verifies the signature of the encoded artifact in the provided
     * repositoryProvider and returns an artifactory for it.
     * Calling this method generally requires prior initialization of the
     * repositoryProvider.
     * Though initialization is a side effect of calling {@link #sign}, this is
     * typically done by decoding the repositoryProvider.
     * <p>
     * It is an implementation detail whether or not verifying {@code null} is
     * supported or not.
     *
     * @param  codec the codec for decoding the artifact.
     * @param  repository the repository for decoding the artifact from.
     * @throws RepositoryIntegrityException if the integrity of the repository
     *         with its encoded artifact has been compromised.
     */
    Artifactory verify(Codec codec, Repository repository) throws Exception;
}
