/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import java.security.*;
import javax.annotation.*;
import javax.annotation.concurrent.Immutable;
import net.java.truelicense.core.codec.*;

/**
 * A controller for storing authenticated objects (artifacts)
 * in a {@link RepositoryModel} with the help of a {@link Codec}.
 * This interface is used by {@linkplain Notary notaries} to
 * {@linkplain Authentication#sign sign} and
 * {@linkplain Authentication#verify verify} artifacts.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public interface Repository {

    /** Returns the repository model. */
    RepositoryModel model();

    /**
     * Encodes and signs the given {@code artifact}
     * and returns an artifactory for decoding it.
     * <p>
     * It is an implementation detail whether or not encoding {@code null} is
     * supported or not.
     *
     * @param  codec the codec for encoding the artifact.
     * @param  engine the signature engine.
     * @param  key the private key.
     * @param  artifact the nullable artifact.
     */
    Artifactory sign(
            Codec codec,
            Signature engine,
            PrivateKey key,
            @Nullable Object artifact)
    throws Exception;

    /**
     * Verifies the signature of the encoded artifact
     * and returns an artifactory for decoding it.
     *
     * @param  codec the codec for decoding the artifact.
     * @param  engine the signature engine.
     * @param  key the public key.
     * @throws IllegalStateException if the state of the model doesn't
     *         match the configuration of the codec or the signature engine.
     * @throws RepositoryIntegrityException if the integrity of this repositoryProvider
     *         with its encoded artifact has been compromised.
     */
    Artifactory verify(
            Codec codec,
            Signature engine,
            PublicKey key)
    throws Exception;
}
